#!/bin/bash
IMPACT="Samba/AD USN rollbacks break the DRS multimaster replication"

exitcode=0

eval "$(ucr shell domainname samba4/role)"

if [ "$samba4_role" != "DC" ] || [ ! -x "$(which ldbsearch)" ]; then
	exit 0
fi

dsa_obj_guid_for_invocationID() {
	local ldif
	ldif=$(ldbsearch -H /var/lib/samba/private/sam.ldb --cross-ncs "(&(objectClass=nTDSDSA)(invocationID=$1))" objectGUID | ldapsearch-wrapper)
	echo "$(sed -n 's/^objectGUID: //p' <<<"$ldif")"
}

ldif=$(ldbsearch -H /var/lib/samba/private/sam.ldb -s base replUpToDateVector --show-binary)
while read -d $'\0' record; do
	dn=$(sed -n 's/^dn: //p' <<<"$record")
	if [ -z "$dn" ]; then
		continue
	fi
	if ! echo $record | grep -q "replUpToDateVector: "; then
		continue
	fi
	source_dsa_invocation_id=$(sed -n 's/^ *source_dsa_invocation_id *: //p' <<<"$record")
	if [ -z "$source_dsa_invocation_id" ]; then
		echo "ERROR: source_dsa_invocation_id not found in replUpToDateVector"
		continue
	fi
	source_dsa_obj_guid=$(dsa_obj_guid_for_invocationID "$source_dsa_invocation_id")
	if [ -z "$source_dsa_obj_guid" ]; then
		echo "ERROR: objectGUID not found for invocationID $source_dsa_invocation_id"
		continue
	fi
	msdcs_alias="$source_dsa_obj_guid._msdcs.$domainname"
	echo -n "Replicating with: $(host -t cname "$msdcs_alias") USN check result: "
	ldif2=$(ldbsearch -H ldap://"$source_dsa_obj_guid._msdcs.$domainname"/ -s base -b '' highestCommittedUSN)
	highest_usn=$(sed -n 's/^ *highest_usn *: .* (\(.*\))/\1/p' <<<"$record")
	highestCommittedUSN=$(sed -n 's/^highestCommittedUSN: //p' <<<"$ldif2")
	if [ "$highestCommittedUSN" -lt "$highest_usn" ]; then
		fqdn=$(dig "$msdcs_alias" cname +short)
		echo "ERROR: USN Rollback detected for $fqdn"
		exitcode=1
	else
		echo OK
	fi
done < <(sed -e 's/^$/\x00/' <<<"$ldif")

exit "$exitcode"
